@@ -1,6 +1,6 @@
 #' Dataset.all
 #'
-#' Retrieves the metadata about all datasets on SolveBio.
+#' Retrieves the metadata about datasets on SolveBio.
 #'
 #' @param ... (optional) Additional query parameters (e.g. page).
 #'
@@ -13,17 +13,18 @@
 #'
 #' @export
 Dataset.all <- function(...) {
-    .request('GET', "v1/datasets", query=list(...))
+    .request('GET', "v2/datasets", query=list(...))
 }
 
+
 #' Dataset.retrieve
 #'
 #' Retrieves the metadata about a specific dataset from SolveBio.
 #'
-#' @param id String The ID or full name of a SolveBio dataset
+#' @param id String The ID of a SolveBio dataset
 #'
 #' @examples \dontrun{
-#' Dataset.retrieve("ClinVar/ClinVar")
+#' Dataset.retrieve("1234567890")
 #' }
 #'
 #' @references
@@ -32,10 +33,10 @@ Dataset.all <- function(...) {
 #' @export
 Dataset.retrieve <- function(id) {
     if (missing(id)) {
-        stop("A dataset ID or name is required.")
+        stop("A dataset ID is required.")
     }
 
-    path <- paste("v1/datasets", paste(id), sep="/")
+    path <- paste("v2/datasets", paste(id), sep="/")
     .request('GET', path=path)
 }
 
@@ -44,10 +45,10 @@ Dataset.retrieve <- function(id) {
 #'
 #' Delete a specific dataset from SolveBio.
 #'
-#' @param id String The ID or full name of a SolveBio dataset
+#' @param id String The ID of a SolveBio dataset
 #'
 #' @examples \dontrun{
-#' Dataset.delete("1")
+#' Dataset.delete("1234567890")
 #' }
 #'
 #' @references
@@ -56,10 +57,10 @@ Dataset.retrieve <- function(id) {
 #' @export
 Dataset.delete <- function(id) {
     if (missing(id)) {
-        stop("A dataset ID or name is required.")
+        stop("A dataset ID is required.")
     }
 
-    path <- paste("v1/datasets", paste(id), sep="/")
+    path <- paste("v2/datasets", paste(id), sep="/")
     .request('DELETE', path=path)
 }
 
@@ -67,12 +68,12 @@ Dataset.delete <- function(id) {
 #' Dataset.data
 #'
 #' Returns one page of documents from a SolveBio dataset and processes the response.
-#' @param id The ID or full name of a SolveBio dataset, or a Dataset object.
+#' @param id The ID of a SolveBio dataset, or a Dataset object.
 #' @param filters (optional) Query filters.
 #' @param ... (optional) Additional query parameters (e.g. limit, offset).
 #'
 #' @examples \dontrun{
-#' Dataset.data("ClinVar/ClinVar")
+#' Dataset.data("1234567890")
 #' }
 #'
 #' @references
@@ -81,7 +82,7 @@ Dataset.delete <- function(id) {
 #' @export
 Dataset.data <- function(id, filters, ...) {
     if (missing(id) | !(class(id) %in% c("Dataset", "numeric", "integer", "character"))) {
-        stop("A dataset ID or name is required.")
+        stop("A dataset ID (or object) is required.")
     }
     if (class(id) == "Dataset") {
         id <- id$id
@@ -99,7 +100,7 @@ Dataset.data <- function(id, filters, ...) {
         body = modifyList(body, list(filters=filters))
     }
 
-    path <- paste("v1/datasets", paste(id), "data", sep="/")
+    path <- paste("v2/datasets", paste(id), "data", sep="/")
 
     tryCatch({
         res <- .request('POST', path=path, body=body)
@@ -114,12 +115,13 @@ Dataset.data <- function(id, filters, ...) {
 #'
 #' Queries a SolveBio dataset and returns an R data frame containing all records (up to 500,000).
 #' Returns a single page of results otherwise (default).
-#' @param id The ID or full name of a SolveBio dataset, or a Dataset object.
+#'
+#' @param id The ID of a SolveBio dataset, or a Dataset object.
 #' @param paginate When set to TRUE, retrieves up to 500,000 records.
 #' @param ... (optional) Additional query parameters (e.g. filters, limit, offset).
 #'
 #' @examples \dontrun{
-#' Dataset.query("ClinVar/ClinVar", paginate=TRUE)
+#' Dataset.query("12345678790", paginate=TRUE)
 #' }
 #'
 #' @references
@@ -132,14 +134,14 @@ Dataset.query <- function(id, paginate=FALSE, ...) {
     params <- list(...)
 
     # Retrieve the first page of results
-    response <- do.call(Dataset.data, c(id=id, params))
+    response <- do.call(Dataset.data, c(id, params))
     df <- response$result
     offset <- response$offset
 
     # continue to make requests for data if pagination is enabled and there are more records
     while (isTRUE(paginate) && !is.null(offset)) {
         params['offset'] <- offset
-        response <- do.call(Dataset.data, c(id=id, params))
+        response <- do.call(Dataset.data, c(id, params))
         df_page <- response$results
         df <- dplyr::bind_rows(df, df_page)
         offset <- response$offset
@@ -164,12 +166,13 @@ Dataset.query <- function(id, paginate=FALSE, ...) {
 #' Dataset.facets
 #'
 #' Retrieves aggregated statistics or term counts for one or more fields in a SolveBio dataset. Returns a list of data frames, one for each requested facet.
-#' @param id The ID or full name of a SolveBio dataset, or a Dataset object.
+#'
+#' @param id The ID of a SolveBio dataset, or a Dataset object.
 #' @param facets A list of one or more field facets.
 #' @param ... (optional) Additional query parameters (e.g. filters, limit, offset).
 #'
 #' @examples \dontrun{
-#' Dataset.facets("ClinVar/Combined", list("clinical_significance", "gene_symbol"))
+#' Dataset.facets("1234567890", list("clinical_significance", "gene_symbol"))
 #' }
 #'
 #' @references
@@ -202,12 +205,13 @@ Dataset.facets <- function(id, facets, ...) {
 #' Dataset.count
 #'
 #' Returns the total number of records for a given SolveBio dataset.
-#' @param id The ID or full name of a SolveBio dataset, or a Dataset object.
+#' @param id The ID of a SolveBio dataset, or a Dataset object.
 #' @param ... (optional) Additional query parameters (e.g. filters, limit, offset).
 #'
 #' @examples \dontrun{
-#' Dataset.count("ClinVar/Variants")
-#' Dataset.count("ClinVar/Variants", filters='[["gene_symbol", "BRCA2"]]')
+#' dataset <- Dataset.get_by_full_path("solvebio:public:/ClinVar/3.7.4-2017-01-30/Variants-GRCh37")
+#' Dataset.count(dataset)
+#' Dataset.count(dataset, filters='[["gene_symbol", "BRCA2"]]')
 #' }
 #'
 #' @references
@@ -228,105 +232,146 @@ Dataset.count <- function(id, ...) {
 #' Dataset.create
 #'
 #' Create an empty SolveBio dataset.
-#' @param depository_version_id The ID of the parent depository verison.
-#' @param name The unique name of the dataset witin the version.
+#' @param vault_id The ID of the vault.
+#' @param vault_parent_object_id The parent object (folder) ID in the vault.
+#' @param name The name of the dataset in the parent folder.
 #' @param ... (optional) Additional dataset attributes.
 #'
 #' @examples \dontrun{
-#' Dataset.create(depository_version_id=1, name="MyDataset")
+#' Dataset.create(vault_id=vault$id, vault_parent_object_id=NULL, name="My Dataset")
 #' }
 #'
 #' @references
 #' \url{https://docs.solvebio.com/}
 #'
 #' @export
-Dataset.create <- function(depository_version_id, name, ...) {
-    if (missing(depository_version_id)) {
-        stop("A depository version ID is required.")
+Dataset.create <- function(vault_id, vault_parent_object_id, name, ...) {
+    if (missing(vault_id)) {
+        stop("A vault ID is required.")
     }
     if (missing(name)) {
-        stop("A name is required.")
+        stop("A dataset name is required")
+    }
+    if (missing(vault_parent_object_id)) {
+        # Use the root of the vault (/)
+        vault_parent_object_id = NULL
     }
 
-    # TODO: Use title if available in (...)
     params = list(
-                  depository_version_id=depository_version_id,
+                  vault_id=vault_id,
+                  vault_parent_object_id=vault_parent_object_id,
                   name=name,
-                  title=name,
                   ...
                   )
 
-    dataset <- .request('POST', path='v1/datasets', query=NULL, body=params)
+    dataset <- .request('POST', path='v2/datasets', query=NULL, body=params)
 
     return(dataset)
 }
 
 
-#' Dataset.get_or_create_by_full_name
+#' Dataset.get_by_full_path
 #'
-#' A helper function to create a dataset on SolveBio using a full name.
-#' @param full_name A valid dataset full name (<depository/version/dataset>).
-#' @param ... (optional) Additional dataset attributes.
+#' A helper function to get a dataset by its full path.
+#'
+#' @param full_path A valid full path to a dataset.
 #'
 #' @examples \dontrun{
-#' Dataset.get_or_create_by_full_name(<FULL NAME>)
+#' Dataset.get_by_full_path("solvebio:public:/ClinVar/3.7.4-2017-01-30/Variants-GRCh37")
 #' }
 #'
 #' @references
 #' \url{https://docs.solvebio.com/}
 #'
 #' @export
-Dataset.get_or_create_by_full_name <- function(full_name, ...) {
-    if (missing(full_name)) {
-        stop("A valid dataset full name is required.")
+Dataset.get_by_full_path <- function(full_path) {
+    object = Object.get_by_full_path(full_path)
+
+    if (is.null(object)) {
+        return(NULL)
     }
 
-    # Split full name
-    parts <- strsplit(full_name, split='/', fixed=TRUE)[[1]]
+    # TODO: This raises an exception on 404, should we return NULL?
+    dataset = do.call(Dataset.retrieve, list(id=object$dataset_id))
+    return(dataset)
+}
 
-    if (length(parts) != 3) {
-        stop("Full names must be in the format: <depository>/<version>/<dataset>")
-    }
 
-    # Check if Dataset already exists
+#' Dataset.get_or_create_by_full_path
+#'
+#' A helper function to get or create a dataset by its full path.
+#'
+#' @param full_path A valid full path to a dataset.
+#' @param ... Additional dataset creation parameters.
+#'
+#' @examples \dontrun{
+#' Dataset.get_or_create_by_full_path("MyVault:/folder/sub-folder/dataset")
+#' }
+#'
+#' @references
+#' \url{https://docs.solvebio.com/}
+#'
+#' @export
+Dataset.get_or_create_by_full_path <- function(full_path, ...) {
+    dataset = NULL
     tryCatch({
-        d <- Dataset.retrieve(full_name)
-        if (!is.null(d)) {
-            return(d)
+        dataset = do.call(Dataset.get_by_full_path, list(full_path))
+        if (!is.null(dataset)) {
+            return(dataset)
         }
     }, error = function(e) {})
 
-    depository_name <- parts[[1]]
-    version_name <- parts[[2]]
-    dataset_name <- parts[[3]]
+    # Create the dataset step-by-step
+    # FIXME: This may break if the path in the vault contains a colon
+    parts <- strsplit(full_path, split=':', fixed=TRUE)[[1]]
 
-    # Get or create Depository
-    depository_obj = tryCatch({
-        depository_obj <- Depository.retrieve(depository_name)
-    }, error = function(e) {
-        depository_obj <- Depository.create(depository_name)
-        return(depository_obj)
-    })
+    if (length(parts) == 3) {
+        account_domain = parts[[1]]
+        vault_name = parts[[2]]
+        object_path = parts[[3]]
+        vault = Vault.get_by_full_path(paste(account_domain, vault_name, sep=":"))
+    }
+    else if (length(parts) == 2) {
+        vault_name = parts[[1]]
+        object_path = parts[[2]]
+        vault = Vault.get_by_full_path(vault_name)
+    }
+    else if (length(parts) == 1) {
+        # TODO: Get or create by (relative) path?
+        stop(sprintf("Invalid full path: %s\n", full_path))
+    }
 
-    # Get or create DepositoryVersion
-    version_obj = tryCatch({
-        version_full_name <- paste(depository_name, version_name, sep="/")
-        version_obj = DepositoryVersion.retrieve(version_full_name)
-    }, error = function(e) {
-        version_obj <- DepositoryVersion.create(
-                                                depository_id=depository_obj$id,
-                                                name=version_name,
-                                                title=version_name
-                                                )
-        return(version_obj)
-    })
+    if (is.null(vault)) {
+        stop(sprintf("Invalid vault in full path: %s\n", full_path))
+    }
 
-    # Create the Dataset
-    dataset_obj <- Dataset.create(
-                                  depository_version_id=version_obj$id,
-                                  name=dataset_name,
-                                  ...
-                                  )
+    if (substring(object_path, 1, 1) != '/') {
+        # Add missing / to object path
+        object_path = paste('/', object_path, sep='')
+    }
 
-    return(dataset_obj)
+    parts <- strsplit(object_path, split='/', fixed=TRUE)[[1]]
+    dataset_name = parts[[length(parts)]]
+    # Remove the filename from the path
+    dirs = utils::head(parts, -1)
+
+    parent_path = paste(dirs, collapse="/")
+    # Get or create the parent folder
+    parent_object = Object.get_by_path(path=parent_path, vault_id=vault$id)
+    if (is.null(parent_object) || (is.data.frame(parent_object) && nrow(parent_object) == 0)) {
+        parent_object = Vault.create_folder(
+                                            id=vault$id,
+                                            path=parent_path,
+                                            recursive=TRUE
+                                            )
+    }
+
+    # Create the dataset under parent_object_id
+    dataset = Dataset.create(
+                             vault_id=vault$id,
+                             vault_parent_object_id=parent_object$id,
+                             name=dataset_name
+                             )
+
+    return(dataset)
 }
