@@ -13,14 +13,14 @@
 #'
 #' @export
 DatasetField.all <- function(...) {
-    .request('GET', "v1/dataset_fields", query=list(...))
+    .request('GET', "v2/dataset_fields", query=list(...))
 }
 
 #' DatasetField.retrieve
 #'
 #' Retrieves the metadata about a specific dataset field.
 #'
-#' @param id String The ID or full name of a dataset field.
+#' @param id String The ID of a dataset field.
 #'
 #' @examples \dontrun{
 #' DatasetField.retrieve(691)
@@ -32,10 +32,10 @@ DatasetField.all <- function(...) {
 #' @export
 DatasetField.retrieve <- function(id) {
     if (missing(id)) {
-        stop("A dataset field ID or name is required.")
+        stop("A dataset field ID is required.")
     }
 
-    path <- paste("v1/dataset_fields", paste(id), sep="/")
+    path <- paste("v2/dataset_fields", paste(id), sep="/")
     .request('GET', path=path)
 }
 
@@ -43,7 +43,7 @@ DatasetField.retrieve <- function(id) {
 #'
 #' Returns the facets for a given dataset field.
 #'
-#' @param id String The ID or full name of a dataset field.
+#' @param id String The ID of a dataset field.
 #' @param ... (optional) Additional query parameters.
 #'
 #' @examples \dontrun{
@@ -56,13 +56,13 @@ DatasetField.retrieve <- function(id) {
 #' @export
 DatasetField.facets <- function(id, ...) {
     if (missing(id) | !(class(id) %in% c("DatasetField", "numeric", "character"))) {
-        stop("A dataset field ID or name is required.")
+        stop("A dataset field ID is required.")
     }
     if (class(id) == "DatasetField") {
         id <- id$id
     }
 
-    path <- paste("v1/dataset_fields", paste(id), "facets", sep="/")
+    path <- paste("v2/dataset_fields", paste(id), "facets", sep="/")
     .request('GET', path=path, query=list(...))
 }
 
@@ -99,7 +99,7 @@ DatasetField.create <- function(dataset_id, name, data_type='auto', ...) {
                   ...
                   )
 
-    dataset_field <- .request('POST', path='v1/dataset_fields', query=NULL, body=params)
+    dataset_field <- .request('POST', path='v2/dataset_fields', query=NULL, body=params)
 
     return(dataset_field)
 }
